#!/bin/sh
set -e

if [ "$#" -ne 4 ]; then
    echo 'Need domain name and mail and old server IP and key ID as argument.'
    false
fi
domain="$1"
mail="$2"
old_server="$3"
gpg_key="$4"

# Install configs, set up firewall.
config_tree_prefix="${HOME}/config/buster"
./install_for_target.sh web website
./copy_dirtree.sh "${config_tree_prefix}/etc_files" "" web website
nft -f /etc/nftables.conf

# Set up letsencrypt certificate. TODO: Is it auto-renewed?
ln -sf /etc/nginx/sites-available/default /etc/nginx/sites-enabled/default
certbot --nginx --agree-tos --redirect --no-eff-email -m "${mail}" -d "${domain}"
rm /etc/nginx/sites-enabled/default

# Set up connection to old server.
cp "${config_tree_prefix}/setup_scripts/prepare_to_meet_server.sh" /home/plom/
chown plom:plom /home/plom/prepare_to_meet_server.sh
su -lc "./prepare_to_meet_server.sh ${old_server}" plom
read -p'Hit Enter when you are done.' ignore
rm /home/plom/prepare_to_meet_server.sh

# Set up repos dir.
mkdir /var/repos
chown plom:plom /var/repos
# To use this dir, "git clone --mirror" repo source paths into it as user plom.
# As user plom, touch git-daemon-export-ok files into it to make the repo
# publically available.
# Outcomment following lines if setting up new / not migrating.
cp "${config_tree_prefix}/setup_scripts/mirror_dir.sh" /home/plom/
chmod a+w /var
su -lc "./mirror_dir.sh ${old_server} /var/repos" plom
chmod a-w /var
rm /home/plom/mirror_dir.sh

# Prepare NGINX and GitWeb config.
sed -i "s/REPLACE_fqdn_ECALPER/${domain}/g" /etc/gitweb.conf
sed -i "s/REPLACE_fqdn_ECALPER/${domain}/g" /etc/nginx/sites-available/website.nginx
ln -s /etc/nginx/sites-available/website.nginx /etc/nginx/sites-enabled/website.nginx

# Set up website. TODO: use non-/var/www dir for better separation to dump site
rm -rf /var/www
mkdir /var/www
chown plom:plom /var/www
# Use next two lines if setting up new / not migrating.
#su -lc "cd /var/repos && git clone --mirror ${old_server}:repos/website" plom
#cp "${config_tree_prefix}/other_files/website_hook_post-receive" /var/repos/website.git/hooks/post-receive
su -lc 'cd /var/www && git clone /var/repos/website.git .' plom

# Add encryption key.
keyservers='sks-keyservers.net/ keys.gnupg.net'
set +e
while true; do
    do_break=0
    for keyserver in $(echo "${keyservers}"); do
        su plom -c "gpg --no-tty --keyserver $keyserver --recv-key ${gpg_key}"
        if [ $? -eq "0" ]; then
            do_break=1
            break
        fi
        echo "Attempt with keyserver ${keyserver} unsuccessful, trying other."
    done
    if [ "${do_break}" -eq "1" ]; then
        break
    fi
done
set -e

# Set up plomlombot.
irclogs_dir=/var/www/html/irclogs
irclogs_pw_dir=/var/www/irclogs_pw
mkdir -p "${irclogs_dir}"
chown -R plom:plom "${irclogs_dir}"
mkdir -p "${irclogs_pw_dir}"
chown -R plom:plom "${irclogs_pw_dir}"
# Use following lines if setting up new / not migrating.
#su -lc "cd /var/repos && git clone --mirror https://plomlompom.com/repos/clone/plomlombot-irc" plom
#su -lc "touch /var/repos/plomlombot-irc.git/git-daemon-export-ok" plom
#cp "${config_tree_prefix}/other_files/plomlombot_hook_post-receive" /var/repos/plomlombot-irc.git/hooks/post-receive

su -lc "git clone /var/repos/plomlombot-irc.git" plom
cp "${config_tree_prefix}/other_files/plomlombot_daemon.sh" /home/plom/
chown plom:plom /home/plom/plomlombot_daemon.sh
# Use upper outcommented instead of lower code if setting up new empty server.
#echo 'bot: plomlombog plomlombog #plomlomtest irc.freenode.net foo bar' >> /home/plom/.plomlombot
#chown plom:plom /home/plom/.plomlombot
cp "${config_tree_prefix}/setup_scripts/mirror_dir.sh" /home/plom/
su -lc "./mirror_dir.sh ${old_server} /home/plom/plomlombot_db" plom
rm /home/plom/mirror_dir.sh
su -lc "scp plom@${old_server}:.plomlombot ~" plom
ssh plom@"${old_server}" "su -lc 'service plomlombot stop'"

systemctl enable plomlombot.service
service plomlombot start

# TODO:
# - commit git-daemon-export-ok directly into the public repos; rename
#   /home/plom/public_repos to /home/plom/repos

service nginx restart
